<?php

namespace Manadev\ProductCollection\Helper;

use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterfaceFactory;
use Magento\InventorySalesApi\Api\GetStockBySalesChannelInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Manadev\ProductCollection\Resources\HelperResource;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Zend_Db;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    const CONFIG_SHOW_OOS_PRODUCTS = 'cataloginventory/options/show_out_of_stock';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $resourceConnection;
    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    protected $product_entity_ids_to_filter = [];
    /**
     * @var HelperResource
     */
    protected $helperResource;
    /**
     * @var Collection
     */
    protected $productCollection;


    public function __construct(
        Context $context,
        StoreManagerInterface $storeManager,
        HelperResource $helperResource,
        Collection $productCollection
    )
    {
        parent::__construct($context);
        $this->scopeConfig = $context->getScopeConfig();
        $this->storeManager = $storeManager;
        $this->helperResource = $helperResource;
        $this->productCollection = $productCollection;
    }

    public function getConfig($config_path)
    {
        return $this->scopeConfig->getValue(
            $config_path,
            ScopeInterface::SCOPE_STORE
        );
    }

    public function getShowOosProducts() {
        return $this->getConfig(self::CONFIG_SHOW_OOS_PRODUCTS);
    }

    /**
     * FOS-545
     *
     * --  ________                          _____ _____
     * --  ___  __ \_____ __________________ __  /___  /_______
     * --  __  /_/ /_  _ \___  /___  /_  __ \_  __/_  __/_  __ \
     * --  _  ____/ /  __/__  /___  /_/ /_/ // /_  / /_  / /_/ /
     * --  /_/      \___/ _____/_____/\____/ \__/  \__/  \____/
     * --
     * @param $facet
     * @return array
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getInStock($facet) {
        $db = $this->productCollection->getConnection();
        $storeManager = $this->storeManager->getStore();
        $storeId = $storeManager->getId();


        $attributeId = $facet->getAttributeId();
        $select = $db->select();

        $selectedOptionIds = $facet->getSelectedOptionIds() ? $facet->getSelectedOptionIds() : [];

        $objectManager = ObjectManager::getInstance();
        if ($this->helperResource->isMultistockAvailable()) {
            $salesChannel = $objectManager->get(SalesChannelInterfaceFactory::class);
            $stockChannel = $objectManager->get(GetStockBySalesChannelInterface::class);

            $channel = $salesChannel->create();
            /** @var SalesChannelInterface $channel */
            $channel->setType(SalesChannelInterface::TYPE_WEBSITE);
            $channel->setCode($storeManager->getWebsite()->getCode());
            $stockInfo = $stockChannel->execute($channel);

            $stockSubquery = '(SELECT (SUM(x.quantity) + so.quantity) FROM %s x WHERE x.sku IN (cpe.sku))';

            $stockQuery = new \Zend_Db_Expr(sprintf($stockSubquery, $this->productCollection->getConnection()->getTableName('inventory_reservation')));

            $select
                ->from(
                    ['eav' => $this->productCollection->getTable('catalog_product_index_eav')],
                    [ 'value' => 'value', 'entity_id'=>'entity_id','source_id' => 'source_id']
                )
                ->joinLeft(
                    ['cpe' => $this->productCollection->getTable('catalog_product_entity')],
                    '`cpe`.`entity_id` = `eav`.`source_id`',
                    ['sku' => 'cpe.sku']
                )
                ->join(
                    ['so' => $this->productCollection->getTable("inventory_stock_{$stockInfo->getStockId()}")],
                    '`so`.`sku` = `cpe`.`sku`',
                    ['so.quantity', 'stock' => $stockQuery]);
            $select
                ->where("attribute_id = {$attributeId} AND store_id = {$storeId} AND so.is_salable > 0")
                ->having('stock > 0');
        } else {
            $select
                ->from(
                    ['eav' => $this->productCollection->getTable('catalog_product_index_eav')],
                    ['is_salable' => 'cssi.stock_status', 'value' => 'value', 'entity_id'=>'entity_id','source_id' => 'source_id']
                )
                ->joinLeft(
                    ['cpe' => $this->productCollection->getTable('catalog_product_entity')],
                    "`cpe`.`entity_id` = `eav`.`source_id`",
                    []
                )
                ->join(
                    ['cssi' => $this->productCollection->getTable('cataloginventory_stock_status_idx')],
                    '`cssi`.`product_id` = `cpe`.`entity_id`',
                    []
                )
                ->where("attribute_id = {$attributeId} AND store_id = {$storeId} AND cssi.stock_status > 0");
        }

        $result = $db->fetchAll($select, [], Zend_Db::FETCH_ASSOC);
        $inStock = [];
        $items = [];
        $showOssProducts = $this->getShowOosProducts();
        foreach ($result as $item) {
            $inStock[] = (int)$item['value'];
            if (!$showOssProducts) {
                if ($selectedOptionIds && in_array($item['value'], $selectedOptionIds)) {
                    $items[] = $item;
                }
            }
        }

        foreach ($items as $item) {
            $this->product_entity_ids_to_filter[] = $item['entity_id']; //configurable
        }

        return array_unique($inStock);
    }

    public function getProductEntityIdsToFilter()
    {
        return $this->product_entity_ids_to_filter;
    }

    public function setProductEntityIdsToFilter($array = array())
    {
        $this->product_entity_ids_to_filter = $array;
    }
}
